﻿#region Copyright © convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="BaseGridViewCustomBindingController.cs.cs" company="convivo GmbH">
//    Copyright © 2013 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Controllers.General
{
    #region usings
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text.RegularExpressions;
    using System.Web.Mvc;

    using DevExpress.Data;
    using DevExpress.Web.Mvc;

    using Fruitmonitoring.Helper.Export;
    using Fruitmonitoring.Helper.HtmlExtensions.AdvancedSearch;
    using Fruitmonitoring.Helper.Utility;
    using Fruitmonitoring.Helper.Utility.ExtensionHelper;
    using Fruitmonitoring.Infrastructure.ActionAttributes;
    using Fruitmonitoring.Infrastructure.Security;
    using Fruitmonitoring.Models.BindingHandlers;

    #endregion

    /// <summary>
    /// use this base controller for all controllers, which handles the data for the gridview in a custom way
    /// </summary>
    public abstract class BaseGridViewCustomBindingController : BaseGridViewController
    {
        #region variables
        /// <summary>
        /// </summary>
        private readonly IGridViewExtensionHelper _GridViewExtensionHelper = new GridViewExtensionHelper();

        /// <summary>
        /// </summary>
        private bool loadGridViewModelFromRequest = true;

        /// <summary>
        /// The gridviews default pagesize
        /// </summary>
        private const int DEFAULTPAGESIZE = 20;

        // additional gridview filter expression (devexpress)
        /// <summary>
        /// </summary>
        private string _gridViewAdditionalFilterExpression;

        /// <summary>
        /// </summary>
        public string GridViewAdditionalFilterExpression
        {
            get
            {
                return _gridViewAdditionalFilterExpression;
            }

            set
            {
                // combine old additional filter and new additional filter
                if (string.IsNullOrEmpty(_gridViewAdditionalFilterExpression))
                {
                    _gridViewAdditionalFilterExpression = value;
                }
                else
                {
                    _gridViewAdditionalFilterExpression += " AND " + value;
                }
            }
        }

        // additional gridview filter expression (xml for stored procedure)
        // obsolete!!!... use "GridViewAdditionalFilterExpression" instead
        /// <summary>
        /// </summary>
        private string _gridViewAdditionalFilterExpressionXml;

        /// <summary>
        /// </summary>
        public string GridViewAdditionalFilterExpressionXml
        {
            get
            {
                return _gridViewAdditionalFilterExpressionXml;
            }

            set
            {
                _gridViewAdditionalFilterExpressionXml = value;
            }
        }

        // fixed gridview filter expression
        /// <summary>
        /// </summary>
        private string _gridViewFixedFilterExpression;

        /// <summary>
        /// </summary>
        public string GridViewFixedFilterExpression
        {
            get
            {
                return _gridViewFixedFilterExpression;
            }

            set
            {
                _gridViewFixedFilterExpression = value;
            }
        }

        /// <summary>
        /// </summary>
        private IBindingHandler bindingHandler { get; set; }

        /// <summary>
        /// </summary>
        private string gridViewId { get; set; }

        /// <summary>
        /// </summary>
        private string gridViewPartialPath { get; set; }

        /// <summary>
        /// </summary>
        private string keyFieldName { get; set; }

        /// <summary>
        /// </summary>
        private string activeControllerName { get; set; }

        /// <summary>
        /// </summary>
        private Type modelType { get; set; }

        /// <summary>
        /// </summary>
        private string defaultSortColumn { get; set; }

        /// <summary>
        /// </summary>
        private ColumnSortOrder defaultSortColumnSortOrder { get; set; }
        #endregion

        #region constructors
        /// <summary>
        /// this constructor mustbe called from child class
        /// </summary>
        /// <param name="bindingHandler">
        /// the explizit custom binding handler implementation
        /// </param>
        /// <param name="gridViewId">
        /// unique gridview retialerCompanyId
        /// </param>
        /// <param name="gridViewPartialPath">
        /// path to the gridview partial view
        /// </param>
        /// <param name="keyFieldName">
        /// name of the property in the model which holds the primary key
        /// </param>
        /// <param name="controllerName">
        /// name of the controller, which should handle the incomming gridview requests
        /// </param>
        /// <param name="modelType">
        /// type of the model in the gridview
        /// </param>
        /// <param name="defaultSortColumn">
        /// default sort column
        /// </param>
        /// <param name="defaultSortColumnSortOrder">
        /// default sort column sort order
        /// </param>
        /// <param name="isAdmin">
        /// set admin flag that can be used in the view
        /// </param>
        /// <param name="isSuperUser">
        /// set super user flag that can be used in the view
        /// </param>
        public BaseGridViewCustomBindingController(
            IBindingHandler bindingHandler,
            string gridViewId,
            string gridViewPartialPath,
            string keyFieldName,
            string controllerName,
            Type modelType,
            string defaultSortColumn,
            ColumnSortOrder defaultSortColumnSortOrder,
            bool isAdmin,
            bool isSuperUser)
            : this(bindingHandler, gridViewId, gridViewPartialPath, keyFieldName, controllerName, modelType, false, false)
        {
            this.defaultSortColumn = defaultSortColumn;
            this.defaultSortColumnSortOrder = defaultSortColumnSortOrder;
        }

        /// <summary>
        /// this constructor must be called from child class
        /// </summary>
        /// <param name="bindingHandler">
        /// the explizit custom binding handler implementation
        /// </param>
        /// <param name="gridViewId">
        /// unique gridview retialerCompanyId
        /// </param>
        /// <param name="gridViewPartialPath">
        /// path to the gridview partial view
        /// </param>
        /// <param name="keyFieldName">
        /// name of the property in the model which holds the primary key
        /// </param>
        /// <param name="controllerName">
        /// name of the controller, which should handle the incomming gridview requests
        /// </param>
        /// <param name="modelType">
        /// type of the model in the gridview
        /// </param>
        /// <param name="isAdmin">
        /// set admin flag that can be used in the view
        /// </param>
        /// <param name="isSuperUser">
        /// set super user flag that can be used in the view
        /// </param>
        public BaseGridViewCustomBindingController(
            IBindingHandler bindingHandler,
            string gridViewId,
            string gridViewPartialPath,
            string keyFieldName,
            string controllerName,
            Type modelType,
            bool isAdmin,
            bool isSuperUser)
            : base(controllerName, gridViewId, isAdmin, isSuperUser)
        {
            this.bindingHandler = bindingHandler;
            this.gridViewId = gridViewId;
            this.gridViewPartialPath = gridViewPartialPath;
            this.keyFieldName = keyFieldName;
            this.modelType = modelType;

            if (System.Web.HttpContext.Current != null)
            {
                bindingHandler.SetMembershipUser(System.Web.HttpContext.Current.Request.RequestContext.HttpContext.Request.RequestContext.HttpContext.User);
            }
        }
        #endregion

        #region actionresults
        /// <summary>
        /// Standard grid callback requests
        /// </summary>
        /// <returns>sample index view</returns>
        [AjaxOrChildActionOnly]
        public override ActionResult GridViewListView()
        {
            return null;
        }

        [AjaxOrChildActionOnly]
        public override ActionResult GridViewListViewInit()
        {
            return null;
        }

        /// <summary>
        /// </summary>
        /// <param name="componentValue">
        /// </param>
        /// <returns>
        /// </returns>
        public int GetPageSizeFromSettings(string componentValue)
        {
            return 0;
        }

        /// <summary>
        /// prepare data for gridview
        /// </summary>
        /// <param name="gridViewModel">
        /// current gridview model
        /// </param>
        /// <param name="sortColumn">
        /// gridview column
        /// </param>
        /// <param name="oldPageSize">
        /// presvious page size
        /// </param>
        /// <returns>
        /// sample index view
        /// </returns>
        public virtual PartialViewResult CustomBindingCore(GridViewModel gridViewModel, GridViewColumnState sortColumn = null, int oldPageSize = -1)
        {
            return null;
        }

        /// <summary>
        /// set default sortColumn
        /// </summary>
        /// <param name="sortColumn">
        /// current sortcolumn reference
        /// </param>
        /// <returns>
        /// sort column with default sorting informations
        /// </returns>
        private GridViewColumnState generateDefaultSortColumn(GridViewColumnState sortColumn)
        {
            return null;
        }

        /// <summary>
        /// Paging operation requests
        /// </summary>
        /// <param name="pager">
        /// gridview pager object
        /// </param>
        /// <returns>
        /// sample index view
        /// </returns>
        [AjaxOrChildActionOnly]
        public ActionResult CustomPaging(GridViewPagerState pager)
        {
            return null;
        }

        /// <summary>
        /// Sorting operation requests
        /// </summary>
        /// <param name="column">
        /// gridview column
        /// </param>
        /// <param name="reset">
        /// ??
        /// </param>
        /// <returns>
        /// sample index view
        /// </returns>
        [AjaxOrChildActionOnly]
        public ActionResult CustomSorting(GridViewColumnState column, bool reset)
        {
            return null;
        }

        /// <summary>
        /// Grouping operation requests
        /// </summary>
        /// <param name="column">
        /// gridview column
        /// </param>
        /// <returns>
        /// sample index view
        /// </returns>
        [AjaxOrChildActionOnly]
        public ActionResult CustomGrouping(GridViewColumnState column)
        {
            return null;
        }

        /// <summary>
        /// Filtering operation requests
        /// </summary>
        /// <param name="column">
        /// gridview column
        /// </param>
        /// <param name="form">
        /// </param>
        /// <returns>
        /// sample index view
        /// </returns>
        [AjaxOrChildActionOnly]
        public ActionResult CustomFiltering(GridViewFilteringState filteringState)
        {
            return null;
        }

        /// <summary>
        /// js: performCallback() function comes in here
        /// also column width changed is handled here
        /// </summary>
        /// <param name="parameters">
        /// parameter dictionary which holds all objects
        /// </param>
        /// <returns>
        /// </returns>
        [AjaxOrChildActionOnly]
        public virtual ActionResult PerformCallbackCustom(Dictionary<string, string> parameters)
        {
            return null;
        }
        #endregion

        #region methods
        /// <summary>
        /// Adds the advanced search filter expression.
        /// </summary>
        /// <param name="advancedSearchFilterExpression">The filter expression.</param>
        private void setAdvancedSearchFilterExpression(string advancedSearchFilterExpression)
        {
        }

        /// <summary>
        /// validate page index and
        /// set new pageindex if necessary
        /// </summary>
        /// <param name="gridViewModel">
        /// current gridviewmodel
        /// </param>
        /// <param name="offset">
        /// current offset
        /// </param>
        /// <param name="oldPageSize">
        /// old page size of the gridview
        /// </param>
        protected void validatePageIndex(ref GridViewModel gridViewModel, ref int offset, int oldPageSize)
        {
        }

        /// <summary>
        /// parse "Ascending" or "Descending" to its enum synonym
        /// </summary>
        /// <param name="orderType">
        /// "Ascending" or "Descending
        /// </param>
        /// <returns>
        /// enum snonym
        /// </returns>
        protected BaseBindingHandlers.DbOrderType parseSortOrder(string orderType)
        {
            return BaseBindingHandlers.DbOrderType.NONE;
        }

        /// <summary>
        /// gets the active sort column name
        /// </summary>
        /// <param name="gridViewModel">
        /// active gridviewmodel
        /// </param>
        /// <param name="sortColumn">
        /// new sortcolumn
        /// </param>
        /// <returns>
        /// column name
        /// </returns>
        protected string generateOrderBy(GridViewModel gridViewModel, GridViewColumnState sortColumn)
        {
            return null;
        }

        /// <summary>
        /// gets the active ordertype
        /// </summary>
        /// <param name="gridViewModel">
        /// active gridviewmodel
        /// </param>
        /// <param name="sortColumn">
        /// new sortcolumn
        /// </param>
        /// <returns>
        /// order type
        /// </returns>
        protected BaseBindingHandlers.DbOrderType generateOrderType(GridViewModel gridViewModel, GridViewColumnState sortColumn)
        {
            return BaseBindingHandlers.DbOrderType.NONE;
        }

        /// <summary>
        /// Creates an emtpy gridview model from settings data.
        /// </summary>
        /// <param name="pageSize">The number of items that the gridview shows at on page. Provide 0 if the gridview should show no items.</param>
        /// <returns>The gridview model.</returns>
        protected GridViewModel CreateGridViewModel(int pageSize)
        {
            return null;
        }

        /// <summary>
        /// initializes the export
        /// this export function is used by the custom binding controller to make sure, that the sorting and filtering is not applied via LINQ
        /// </summary>
        /// <param name="model">
        /// </param>
        /// <param name="exportType">
        /// type of the export
        /// </param>
        /// <param name="columnNames">
        /// names of all columnNamesArray (separated by "[,]")
        /// </param>
        /// <param name="columnTexts">
        /// texts of all columnNamesArray (separated by "[,]")
        /// </param>
        /// <param name="filter">
        /// gridview filter expression
        /// </param>
        /// <param name="sortColumnName">
        /// sorted column name
        /// </param>
        /// <param name="sortOrder">
        /// sort order "Descending" or "Ascending"
        /// </param>
        /// <param name="applySorting">
        /// use active sorting?
        /// </param>
        /// <param name="applyFilter">
        /// use active filtering?
        /// </param>
        /// <param name="exportOptions">
        /// export options object
        /// </param>
        /// <returns>
        /// excel file
        /// </returns>
        protected override ActionResult Export<T>(
            IQueryable<T> model,
            string exportType,
            string columnNames,
            string columnTexts,
            string filter = "",
            string sortColumnName = "",
            string sortOrder = "Descending",
            bool applySorting = false,
            bool applyFilter = false,
            ExportOptions exportOptions = null)
        {
            return null;
        }

        /// <summary>
        /// </summary>
        /// <param name="key">
        /// </param>
        /// <param name="value">
        /// </param>
        protected void SetEntryInBindingHandlerDictionary(string key, object value)
        {
        }

        /// <summary>
        /// </summary>
        /// <param name="key">
        /// </param>
        protected void RemoveEntryInBindingHandlerDictionary(string key)
        {
        }
        #endregion
    }
}