﻿#region Copyright © convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="EMailOptions.cs.cs" company="convivo GmbH">
//    Copyright © 2014 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Helper.Utility.Email
{
    #region usings
    using System.Web.Mvc;

    #endregion

    /// <summary>
    /// Holds email options that can set for sending emails.
    /// </summary>
    public class EMailOptions
    {
        #region Properties
        /// <summary>
        /// The current controller context.
        /// </summary>
        public ControllerContext ControllerContext { get; set; }

        // the emails attachment data
        /// <summary>
        /// Gets or sets the file content.
        /// </summary>
        public byte[] FileContent { get; set; }

        // the emails attachment filename
        /// <summary>
        /// Gets or sets the file name.
        /// </summary>
        public string FileName { get; set; }

        // is the provided "mailTemplate" a path to a view or a string content
        /// <summary>
        /// Gets or sets a value indicating whether is mail template a path.
        /// </summary>
        public bool IsMailTemplateAPath { get; set; }

        // the email body view-template or string content
        /// <summary>
        /// Gets or sets the mail template.
        /// </summary>
        public string MailTemplate { get; set; }

        // recipients of the mail
        /// <summary>
        /// Gets or sets the recipients.
        /// </summary>
        public string Recipients { get; set; }

        // recipients BC for the mail
        /// <summary>
        /// Gets or sets the recipients bc.
        /// </summary>
        public string RecipientsBC { get; set; }

        // recipients CC for the mail
        /// <summary>
        /// Gets or sets the recipients cc.
        /// </summary>
        public string RecipientsCC { get; set; }

        // the mails subject
        /// <summary>
        /// Gets or sets the subject.
        /// </summary>
        public string Subject { get; set; }

        // a temporary dictionary that can filled with custom content and can used by the mailTemplate
        /// <summary>
        /// Gets or sets the temp dict.
        /// </summary>
        public TempDataDictionary TempDict { get; set; }

        // a  dictionary that can filled with custom content and can used by the mailTemplate
        // set ViewDict.Model to provide a model object to the view
        /// <summary>
        /// Gets or sets the view dict.
        /// </summary>
        public ViewDataDictionary ViewDict { get; set; }
        #endregion

        #region Constructors and Destructors
        /// <summary>
        /// Initializes a new instance of the <see cref="EMailOptions"/> class.
        /// use this constructor if an controller context exists.
        /// </summary>
        /// <param name="subject">
        /// Subject for this email.
        /// </param>
        /// <param name="recipients">
        /// Recipients for this email.
        /// </param>
        /// <param name="mailTemplate">
        /// The email template.
        /// </param>
        /// <param name="isMailTemplateAPath">
        /// True if the template is a path to the view, if the template is a string content provide false.
        /// </param>
        /// <param name="controllerContext">
        /// The current controller context.
        /// </param>
        /// <param name="viewDict">
        /// A viewdata dictionary. that data can used by the view to get individial informations. set viewDict.Model if the view expects a model object.
        /// </param>
        /// <param name="tempDict">
        /// A temporary dictionary. that data can used by the view to get individual informations.
        /// </param>
        public EMailOptions(
            string subject,
            string recipients,
            string mailTemplate,
            bool isMailTemplateAPath,
            ControllerContext controllerContext,
            ViewDataDictionary viewDict,
            TempDataDictionary tempDict)
            : this(subject, recipients, mailTemplate, isMailTemplateAPath, viewDict)
        {
            ControllerContext = controllerContext;
            TempDict = tempDict;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="EMailOptions"/> class.
        /// use this constructor if no controller context exists
        /// the emailer tries to parse the template without an constructor.
        /// </summary>
        /// <param name="subject">
        /// Subject for this email.
        /// </param>
        /// <param name="recipients">
        /// Recipients for this email.
        /// </param>
        /// <param name="mailTemplate">
        /// The email template.
        /// </param>
        /// <param name="isMailTemplateAPath">
        /// True if the template is a path to the view, if the template is a string content provide false.
        /// </param>
        /// <param name="viewDict">
        /// Set viewDict.Model if the view expects a model object. you can NOT use this object to provide custom data to the view.
        /// </param>
        public EMailOptions(string subject, string recipients, string mailTemplate, bool isMailTemplateAPath, ViewDataDictionary viewDict)
        {
            Subject = subject;
            Recipients = recipients;
            MailTemplate = mailTemplate;
            IsMailTemplateAPath = isMailTemplateAPath;
            ViewDict = viewDict;
        }
        #endregion

        #region Fields

        // the content type for the mail. you can provide normal text or html text
        /// <summary>
        /// The content type.
        /// </summary>
        public CONTENTTYPE ContentType = CONTENTTYPE.HTML;

        // the email layout file path for static content
        // this is needed if no controller context is provided
        /// <summary>
        /// The e mail static layout path.
        /// </summary>
        public string EMailStaticLayoutPath = "~/Views/Shared/Fruitmonitoring/Emails/_emailLayoutStatic.cshtml";

        // the importance for the mail
        /// <summary>
        /// The importance.
        /// </summary>
        public IMPORTANCE Importance = IMPORTANCE.NORMAL;

        // the sensitivity for the mail
        /// <summary>
        /// The sensitivity.
        /// </summary>
        public SENSITIVITY Sensitivity = SENSITIVITY.NORMAL;
        #endregion

        #region Enums

        // allowed content types
        /// <summary>
        /// The contenttype.
        /// </summary>
        public enum CONTENTTYPE
        {
            /// <summary>
            /// The html.
            /// </summary>
            HTML,

            /// <summary>
            /// The text.
            /// </summary>
            TEXT
        }

        // allowd importances
        /// <summary>
        /// The importance.
        /// </summary>
        public enum IMPORTANCE
        {
            /// <summary>
            /// The low.
            /// </summary>
            LOW,

            /// <summary>
            /// The normal.
            /// </summary>
            NORMAL,

            /// <summary>
            /// The high.
            /// </summary>
            HIGH
        }

        // allowed sensitivities
        /// <summary>
        /// The sensitivity.
        /// </summary>
        public enum SENSITIVITY
        {
            /// <summary>
            /// The normal.
            /// </summary>
            NORMAL
        }
        #endregion

        #region Public Methods and Operators
        /// <summary>
        /// Get a contenttype value that can be read by the send mail procedure.
        /// </summary>
        /// <param name="importance">
        /// </param>
        /// <returns>
        /// The <see cref="string"/>.
        /// </returns>
        public string GetContentType()
        {
            return null;
        }

        /// <summary>
        /// Get a importance value that can be read by the send mail procedure.
        /// </summary>
        /// <param name="importance">
        /// </param>
        /// <returns>
        /// The <see cref="string"/>.
        /// </returns>
        public string GetImportance()
        {
            return null;
        }

        /// <summary>
        /// Get a sensititvity value that can be read by the send mail procedure.
        /// </summary>
        /// <param name="importance">
        /// </param>
        /// <returns>
        /// The <see cref="string"/>.
        /// </returns>
        public string GetSensitivity()
        {
            return null;
        }
        #endregion
    }
}