﻿#region Copyright © convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="RequiredIfPropertyHasValue.cs.cs" company="convivo GmbH">
//    Copyright © 2014 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Helper.Validation
{
    #region usings
    using System;
    using System.Collections.Generic;
    using System.ComponentModel.DataAnnotations;
    using System.Web.Mvc;

    using Fruitmonitoring.Helper.Utility;

    #endregion

    /// <summary>
    /// RequiredIf attribute. If "propertyName" has "desiredValue", the annotated property will be required to match the value in "pattern".
    /// </summary>
    public class RequiredIfPropertyHasValue : ValidationAttribute, IClientValidatable
    {
        #region Properties
        /// <summary>
        /// Gets or sets the desired value.
        /// </summary>
        protected object DesiredValue { get; set; }

        /// <summary>
        /// Gets or sets the property name.
        /// </summary>
        protected string PropertyName { get; set; }

        /// <summary>
        /// Gets or sets a value indicating whether to lower.
        /// </summary>
        protected bool ToLower { get; set; }

        #region Fields
        /// <summary>
        /// The _regular expression attribute.
        /// </summary>
        protected RegularExpressionAttribute _regularExpressionAttribute;
        #endregion

        #endregion

        #region Constructors and Destructors
        /// <summary>
        /// Initializes a new instance of the <see cref="RequiredIfPropertyHasValue"/> class.
        /// RequiredIf attribute. If "propertyName" has "desiredValue", the annotated property will be required to match the value in "pattern".
        /// </summary>
        /// <param name="propertyName">
        /// Property in the model that must have the desired value.
        /// </param>
        /// <param name="desiredValue">
        /// Value the property must have.
        /// </param>
        /// <param name="pattern">
        /// The annotated property's value must match this pattern if the other property has the desired value.
        /// </param>
        /// <param name="toLower">
        /// Converts stuff to ToLower() before comparing.
        /// </param>
        public RequiredIfPropertyHasValue(string propertyName, object desiredValue, string pattern, bool toLower = false)
        {
            _regularExpressionAttribute = new RegularExpressionAttribute(pattern);

            PropertyName = propertyName;
            DesiredValue = desiredValue;
            ToLower = toLower;
        }
        #endregion

        #region IClientValidatable Members

        #region Public Methods and Operators

        #region IClientValidatable Member
        /// <summary>
        /// Returns rules for the Client validition.
        /// </summary>
        /// <param name="metadata">
        /// ModelMetadata.
        /// </param>
        /// <param name="context">
        /// Validation context.
        /// </param>
        /// <returns>
        /// Meta data for the client validation.
        /// </returns>
        public IEnumerable<ModelClientValidationRule> GetClientValidationRules(ModelMetadata metadata, ControllerContext context)
        {
            var rule = new ModelClientValidationRule { ErrorMessage = FormatErrorMessage(metadata.GetDisplayName()), ValidationType = "requiredif", };

            string depProp = BuildDependentPropertyId(metadata, context as ViewContext);

            // find the value on the control we depend on;
            // if it's a bool, format it javascript style 
            // (the default is True or False!)
            string targetValue = (DesiredValue ?? string.Empty).ToString();
            if (DesiredValue is bool)
            {
                targetValue = targetValue.ToLower();
            }

            rule.ValidationParameters.Add("dependentproperty", depProp);
            rule.ValidationParameters.Add("targetvalue", targetValue);
            rule.ValidationParameters.Add("regexpression", _regularExpressionAttribute.Pattern);
            yield return rule;
        }
        #endregion

        #endregion

        #endregion

        #region Methods
        /// <summary>
        /// The build dependent property id.
        /// </summary>
        /// <param name="metadata">
        /// The metadata.
        /// </param>
        /// <param name="viewContext">
        /// The view context.
        /// </param>
        /// <returns>
        /// The <see cref="string"/>.
        /// </returns>
        private string BuildDependentPropertyId(ModelMetadata metadata, ViewContext viewContext)
        {
            string depProp = viewContext.ViewData.TemplateInfo.GetFullHtmlFieldId(PropertyName);

            // unfortunately this will have the name of the current field appended to the beginning,
            // because the TemplateInfo's context has had this fieldname appended to it. Instead, we
            // want to get the context as though it was one level higher (i.e. outside the current property,
            // which is the containing object, and hence the same level as the dependent property.
            var thisField = metadata.PropertyName + "_";
            if (depProp.StartsWith(thisField))
            {
                // strip it off again
                depProp = depProp.Substring(thisField.Length);
            }

            return depProp;
        }
        #endregion

        #region methods
        /// <summary>
        /// The compare values.
        /// </summary>
        /// <param name="propertyValuestr">
        /// The property valuestr.
        /// </param>
        /// <param name="desiredValueString">
        /// The desired value string.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        protected virtual bool CompareValues(string propertyValuestr, string desiredValueString)
        {
            return false;
        }

        /// <summary>
        /// Check if field is required in this context.
        /// </summary>
        /// <param name="value">
        /// Current value.
        /// </param>
        /// <param name="validationContext">
        /// Validation context.
        /// </param>
        /// <returns>
        /// ValidationResult.Success, otherwise a ValidationResult.
        /// </returns>
        protected override ValidationResult IsValid(object value, ValidationContext validationContext)
        {
            return null;
        }
        #endregion
    }
}